@@ -708,6 +708,7 @@ LGBM_SE GPB_CreateREModel_R(LGBM_SE ndata,
 	LGBM_SE num_gp_rand_coef,
 	LGBM_SE cov_fct,
 	LGBM_SE cov_fct_shape,
+	LGBM_SE cov_fct_taper_range,
 	LGBM_SE vecchia_approx,
 	LGBM_SE num_neighbors,
 	LGBM_SE vecchia_ordering,
@@ -718,11 +719,27 @@ LGBM_SE GPB_CreateREModel_R(LGBM_SE ndata,
 	LGBM_SE call_state) {
 	R_API_BEGIN();
 	REModelHandle handle = nullptr;
-	CHECK_CALL(GPB_CreateREModel(R_AS_INT(ndata), R_INT_PTR(cluster_ids_data), R_CHAR_PTR(re_group_data),
-		R_AS_INT(num_re_group), R_REAL_PTR(re_group_rand_coef_data), R_INT_PTR(ind_effect_group_rand_coef), R_AS_INT(num_re_group_rand_coef),
-		R_AS_INT(num_gp), R_REAL_PTR(gp_coords_data), R_AS_INT(dim_gp_coords), R_REAL_PTR(gp_rand_coef_data), R_AS_INT(num_gp_rand_coef),
-		R_CHAR_PTR(cov_fct), R_AS_DOUBLE(cov_fct_shape), R_AS_BOOL(vecchia_approx), R_AS_INT(num_neighbors), R_CHAR_PTR(vecchia_ordering), R_CHAR_PTR(vecchia_pred_type),
-		R_AS_INT(num_neighbors_pred), R_CHAR_PTR(likelihood), &handle));
+	CHECK_CALL(GPB_CreateREModel(R_AS_INT(ndata),
+		R_INT_PTR(cluster_ids_data),
+		R_CHAR_PTR(re_group_data),
+		R_AS_INT(num_re_group),
+		R_REAL_PTR(re_group_rand_coef_data),
+		R_INT_PTR(ind_effect_group_rand_coef),
+		R_AS_INT(num_re_group_rand_coef),
+		R_AS_INT(num_gp),
+		R_REAL_PTR(gp_coords_data),
+		R_AS_INT(dim_gp_coords),
+		R_REAL_PTR(gp_rand_coef_data),
+		R_AS_INT(num_gp_rand_coef),
+		R_CHAR_PTR(cov_fct),
+		R_AS_DOUBLE(cov_fct_shape),
+		R_AS_DOUBLE(cov_fct_taper_range),
+		R_AS_BOOL(vecchia_approx),
+		R_AS_INT(num_neighbors),
+		R_CHAR_PTR(vecchia_ordering),
+		R_CHAR_PTR(vecchia_pred_type),
+		R_AS_INT(num_neighbors_pred),
+		R_CHAR_PTR(likelihood), &handle));
 	R_SET_PTR(out, handle);
 	R_API_END();
 }
@@ -994,7 +1011,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"LGBM_BoosterSaveModel_R"          , (DL_FUNC)&LGBM_BoosterSaveModel_R          , 5},
   {"LGBM_BoosterSaveModelToString_R"  , (DL_FUNC)&LGBM_BoosterSaveModelToString_R  , 8},
   {"LGBM_BoosterDumpModel_R"          , (DL_FUNC)&LGBM_BoosterDumpModel_R          , 7},
-  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 21},
+  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 22},
   {"GPB_REModelFree_R"                , (DL_FUNC)&GPB_REModelFree_R                , 2},
   {"GPB_SetOptimConfig_R"             , (DL_FUNC)&GPB_SetOptimConfig_R             , 14},
   {"GPB_SetOptimCoefConfig_R"         , (DL_FUNC)&GPB_SetOptimCoefConfig_R         , 7},
